/*!
  \file   apriori/OneByOneSupportCounter.hpp
  \brief  OnyByOneSupportCounter treats transactions individually, i.e.  it
  reads the transactions one by one and calls the support count
  method for each of them

  \author Ferenc BODON
  \date   2005-05-04
*/
#ifndef OneByOneSupportCounter_HPP
#define OneByOneSupportCounter_HPP

#include "common.hpp"
#include <vector>
//#include <cstdio>
//#include <iterator>	//for test



/** OnyByOneSupportCounter treats transactions individually, i.e.  it
    reads the transactions one by one and calls the support count
    method for each of them. The class that actually does the "sub
    support count" is given in the constructor and its type is given
    by the third template parameter.  
    \param DATA_STRUCTURE the data structure that stores the candidates 
    \param TR a Transaction_Reader 
    \param SC the support counter that actually does the support counting. 
    It has to implement the  SupportCounterInterface interface.
 */
template <class DATA_STRUCTURE, class TR, class SC = Dummy>
class OneByOneSupportCounter
{
   private:

      DATA_STRUCTURE& data_structure;
      TR& transaction_reader;
      SC support_counter;

   public:
      OneByOneSupportCounter( DATA_STRUCTURE& data_structure,
			      TR& transaction_reader) :
	 data_structure(data_structure), 
	 transaction_reader(transaction_reader){}
      void countSupport(const unsigned int candidate_size);
};

template <class DATA_STRUCTURE, class TR, class SC> inline void 
OneByOneSupportCounter<DATA_STRUCTURE, TR, SC>::
countSupport(const unsigned int candidate_size )
{
   assert( candidate_size > 2 );
   transaction_reader.rewind();
   std::vector<item_t> transaction;
   counter_t multiplicity;
   while( (multiplicity = transaction_reader.nextTransactionBIS(
	      transaction)) !=0 )
      if( candidate_size <= transaction.size() )
	 support_counter.updateCounters(
	    data_structure, transaction, 
	    candidate_size, multiplicity);

}
#endif
