/*!  
  \file apriori/bodon/SupportCounterInterface.hpp 
  \brief This is an interface of a support counter, whose task is 
  to increment the counters of those candidates that are included 
  in the given transaction.

  \author Ferenc BODON
  \date 2005-05-19
*/
#ifndef SupportCounterInterface_HPP
#define SupportCounterInterface_HPP

#include "common.hpp"
#include <vector>


namespace Bodon
{
/**
   This is an interface of a support counter, whose task is 
  to increment the counters of those candidates that are included 
  in the given transaction.

  For an overview about support counters the reader is referred to 
  the paper 
  <ul>
  <li>"Surprising results of trie-based FIM algorithms" from Ferenc Bodon.
  It can be downloaded from the site 
  (http://www.cs.bme.hu/~bodon/kozos/papers/bodon.pdf) 
  in the case of frequent itemset mining</li>

  <li>"A Trie-based APRIORI Implementation for
  Mining Frequent Item sequences" from Ferenc Bodon. It can be
  downloaded from the webpage of OSDM'05 workshop of ACM SIGKDD 
  (http://www.cs.rpi.edu/~zaki/OSDM05/papers/p56-bodon.pdf) in the case 
  frequent itemsequence mining</li>
  </ul>
 */
   template <class MAIN_DATA_STRUCTURE, class SEC_DATA_STRUCTURE = Dummy>
   class SupportCounterInterface
   {
      public:
	 SupportCounterInterface( ){}

	 /*! Increases the counter of those candidates that are contained 
	   by the given basket.

	   \param main_trie the root of the candidate trie
	   \param transaction the transaction
	   \param candidate_size the size of the candidates
	   \param counter_incr the amount the counters of the contained 
	   candidate have to be increased with
	 */
	 void updateCounters(
	    MAIN_DATA_STRUCTURE& main_trie, const std::vector<item_t>& transaction,
	    item_t candidate_size, const counter_t counter_incr)
	 {
	    findCandidates( 
	       &main_trie, transaction.end()-candidate_size+1, 
	       transaction.begin(), candidate_size,
	       counter_incr ){}
	 }
	 
   };
}
