/*!  
  \file apriori/bodon/trie/trie_manipulators/sequence/support_counter/SupportCounterLookupEdge.hpp 
  \brief This is a support counter (i.e. it implements SupportCounterInterface), 
  which takes the elements of the transactions one-by-one and checks if there exists
  an edge with such label.

  \author Ferenc BODON
  \date 2005-05-19
*/
#ifndef SupportCounterLookupEdge_HPP
#define SupportCounterLookupEdge_HPP


#include "common.hpp"
#include <vector>
#include "apriori/bodon/Leaf.hpp"


namespace Bodon
{
namespace sequence
{
   template <class TRIE>
   class SupportCounterLookupEdge
   {
      public:
	 SupportCounterLookupEdge( ){}

	 void updateCounters(
	    TRIE& main_trie, const std::vector<item_t>& transaction,
	    item_t candidate_size, const counter_t counter_incr)
	 {
	    findCandidates( 
	       &main_trie, transaction.end()-candidate_size+1, 
	       transaction.begin(), candidate_size,
	       counter_incr );
	 }
      protected:
	 typedef Leaf LEAF;
	 /// Increases the counter of those candidates that are contained 
	 /// by the given basket.
	 void findCandidates( TRIE* subtrie,
	    std::vector<item_t>::const_iterator it_transaction_upper_bound,
	    std::vector<item_t>::const_iterator it_transaction, 
	    item_t step_to_candidate,
	    const counter_t counter_incr );

   };

   template <class TRIE> void 
   SupportCounterLookupEdge<TRIE>::findCandidates( 
      TRIE* subtrie, std::vector<item_t>::const_iterator 
      it_transaction_upper_bound,
      std::vector<item_t>::const_iterator it_transaction, 
      item_t step_to_candidate, const counter_t counter_incr )
   {
      --step_to_candidate;
      std::vector<bool>::size_type largestEdgelabel_p1 = 
	 subtrie->edgelist.largestEdgelabel()+1;
      std::vector<bool>::size_type smallestEdgelabel = 
	 subtrie->edgelist.smallestEdgelabel();
      std::vector<bool> pattern(largestEdgelabel_p1 - smallestEdgelabel, true);
      void* subsubtrie;
      for(; it_transaction != it_transaction_upper_bound; ++it_transaction)
	 if(*it_transaction < largestEdgelabel_p1 && 
	    *it_transaction >= smallestEdgelabel &&
	    pattern[*it_transaction - smallestEdgelabel])
	 {
	    subtrie->edgelist.lookupNocheck(*it_transaction, subsubtrie);
	    if(subsubtrie)
	    {
	       if( step_to_candidate ) 
		  findCandidates( static_cast<TRIE*>(subsubtrie),
				  it_transaction_upper_bound + 1, it_transaction+1, 
				  step_to_candidate, counter_incr);	       
	       else
		  static_cast<LEAF*>(subsubtrie)->increaseCounter( counter_incr);
	    }
	    pattern[*it_transaction - smallestEdgelabel] = false;
	 }
   }
}
}
#endif
