/*!
  \file   io/db_cache/transaction_shrinker/RBTreeDBCacheInsertThenDestroySimple.hpp
  \brief  A simple database cacher that enables transaction shrinking.

  \author Ferenc BODON
  \date   2006-05-04
*/
#ifndef RBTreeDBCacheInsertThenDestroySimple_HPP
#define RBTreeDBCacheInsertThenDestroySimple_HPP

#include <vector>
#include <map>
#include "io/db_cache/transaction_shrinker/RBTreeDBCacheInsertThenDestroyBase.hpp"

namespace Bodon
{
/**
   A simple database cacher that enables transaction shrinking.

   When inserting the modified transactions into the database cacher,
   we simply go through the tree that stores the modified transactions
   and insert each of them into the cacher by using a look-up. This is
   not a fast solution, because we do not make use of the fact that
   both trees are actually sorted associative containers.
 */
   template< class T_R, class BIS = std::vector<item_t> >
   class RBTreeDBCacheInsertThenDestroySimple : 
	 public RBTreeDBCacheInsertThenDestroyBase<T_R, BIS>
   {
      public:
	 typedef typename RBTreeDBCacheInsertThenDestroyBase<T_R, BIS>
	 ::params_t params_t;

	 RBTreeDBCacheInsertThenDestroySimple( const params_t* par )
	    : RBTreeDBCacheInsertThenDestroyBase<T_R, BIS>(par){}

	 void rewind()
	 {
	    for( typename std::map<BIS, counter_t>::iterator it_temp = 
		    RBTreeDBCacheInsertThenDestroyBase<T_R, BIS>::temp_rb_tree.begin(); 
		 it_temp != RBTreeDBCacheInsertThenDestroyBase<T_R, BIS>::
		    temp_rb_tree.end(); ++it_temp)
	       RBTreeDBCache<T_R, BIS>::rb_tree[it_temp->first] += 
		  it_temp->second;
	    RBTreeDBCacheInsertThenDestroyBase<T_R, BIS>::rewind();
	 }
   };

}
#endif
